# File tres/Scripts/tresfib.cl
# By Doug Mink
# October 2, 2007

# TRESFIB -- Get TRES fiber contents 

procedure tresfib (spectra)

string spectra="\*.fits"	{prompt="Files to process"}

string indir=""		{prompt="Input directory for data files"}
string outdir=""	{prompt="Output directory for tables"}
int nfiles=0		{prompt="Number of files"}
bool addfile=no		{prompt="Add to database (yes) or use old (no)"}
bool listflat=yes	{prompt="List FLAT spectra by category"}
bool listcomp=yes	{prompt="List COMP spectra by category"}
bool addfib=yes		{prompt="Add FIBi keywords to spectra (yes or no)"}
bool deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print debugging information (yes or no)"}

begin
    bool vb, db, dt
    file imf, tempilist, tempkey, newkey
    file listfile, ddir
    file flats12,flats3,flatsb12,flatsb3,flatm12,flatm3,flatmb12,flatmb3
    file comps12,comps3,compsb12,compsb3,compm12,compm3,compmb12,compmb3
    string fib1, fib2, fib3
    int ns, nspec
    string args, imagetype, object, specs, sspecm, bin1, bin2, exptime
    string filename, imtype, peri, ccdsum, fibsize, osswitch, comptime
    string ts1,ts2,ts3,ts4,ts5, tline1, tline2, dline1, dline2
    char tchar
    real ctime, etime

    vb = verbose
    db = debug
    dt = deltemp
    fixdir (outdir)
    ddir = fixdir.newdir
    specs = spectra

    if (!deftask ("gethead")) {
	wcstools (motd=no)
	}

# Make sure output directory is there
    if (vb) {
	print ("TRESLIST: DestDir = ", ddir)
	}
    if ( ddir != "" &&  ddir != "" && !access (ddir)) {
	mkdir (ddir)
	}

#  Make temporary list of input spectrum files
    tempilist = ddir // mktemp ("tempilist")
    tchar = substr (specs,1,1)
    if (tchar == '@') {
	listfile = substr (specs,2,100)
	copy (listfile, tempilist)
	}
    else {
	files (specs, >tempilist)
	}
    newkey = ddir // mktemp ("tempkey")
    if (access (newkey))
	delete (newkey)
    dline1 = "filename     	type	object	peri	bin	fibsize	osswitch	exptime	comptime	fib1	fib2	fib3"
    dline2 = "-----------	-------	------	----	---	-------	--------	-------	--------	----	----	----"
    
# Generate master table of object spectrum keywords or use existing one
    tempkey = ddir // "speckey.db"
    if (!access (tempkey) || addfile) {
	if (listflat) {
	    flats12 = "flats12.list"
	    flats3 = "flats3.list"
	    flatsb12 = "flatsb12.list"
	    flatsb3 = "flatsb3.list"
	    flatm12 = "flatm12.list"
	    flatm3 = "flatm3.list"
	    flatmb12 = "flatmb12.list"
	    flatmb3 = "flatmb3.list"
	    if (!addfile) {
		if (access (flats12))
		    delete (flats12)
		if (access (flats3))
		    delete (flats3)
		if (access (flatsb12))
		    delete (flatsb12)
		if (access (flatsb3))
		    delete (flatsb3)
		if (access (flatm12))
		    delete (flatm12)
		if (access (flatm3))
		    delete (flatm3)
		if (access (flatmb12))
		    delete (flatmb12)
		if (access (flatmb3))
		    delete (flatmb3)
	    }
	    }
	if (listcomp) {
	    comps12 = "comps12.list"
	    comps3 = "comps3.list"
	    compsb12 = "compsb12.list"
	    compsb3 = "compsb3.list"
	    compm12 = "compm12.list"
	    compm3 = "compm3.list"
	    compmb12 = "compmb12.list"
	    compmb3 = "compmb3.list"
	    if (!addfile) {
		if (access (comps12))
		    delete (comps12)
		if (access (comps3))
		    delete (comps3)
		if (access (compsb12))
		    delete (compsb12)
		if (access (compsb3))
		    delete (compsb3)
		if (access (compm12))
		    delete (compm12)
		if (access (compm3))
		    delete (compm3)
		if (access (compmb12))
		    delete (compmb12)
		if (access (compmb3))
		    delete (compmb3)
		}
	    }
	if (vb)
	    print ("TRESLIST: Making master keyword file ",tempkey)
	ns = 0
	if (!addfile) {
	    print (dline1, >tempkey)
	    print (dline2, >>tempkey)
	    }
	if (vb) {
	    print (dline1)
	    print (dline2)
	    }
	list = tempilist
	while (fscan (list, imf) != EOF) {
	    ns = ns + 1
	    args = imf // " -n 1 -au imagetyp object peri ccdsum fibsize osswitch exptime comptime"
	    gethead (args) | scan (filename, imtype, object, peri, bin1,bin2, fibsize, osswitch, exptime, comptime)
	    ctime = real (comptime)
	    etime = real (exptime)
	    fib1 = "___"
	    fib2 = "___"
	    fib3 = "___"
	    if (imtype == "FLAT") {
		if (peri == "IN") {
		    fib1 = "FLAT"
		    fib2 = "FLAT"
		    fib3 = "OFF"
		    }
		else {
		    fib1 = "OFF"
		    fib2 = "OFF"
		    fib3 = "FLAT"
		    }
		}
	    else if (imtype == "OBJECT") {
		if (object == "COMP") {
		    if (peri == "IN") {
			fib1 = "COMP"
			fib2 = "COMP"
			fib3 = "OFF"
			}
		    else {
			fib1 = "OFF"
			fib2 = "OFF"
			fib3 = "COMP"
			}
		    }
		else if (ctime > 0.0) {
		    if (peri == "OUT") {
			if (osswitch == "YES") {
			    fib1 = "SKY"
			    fib2 = "OBJ"
			    fib3 = "COMP"
			    }
			else {
			    fib1 = "OBJ"
			    fib2 = "SKY"
			    fib3 = "COMP"
			    }
			}
		    else {
		        fib1 = "OFF"
		        fib2 = "OFF"
		        fib3 = "OFF"
		        }
		    }
		else {
		    if (peri == "OUT") {
			if (osswitch == "YES") {
			    fib1 = "SKY"
			    fib2 = "OBJ"
			    fib3 = "OFF"
			    }
			else {
			    fib1 = "OBJ"
			    fib2 = "SKY"
			    fib3 = "OFF"
			    }
			}
		    else {
		        fib1 = "OFF"
		        fib2 = "OFF"
		        fib3 = "OFF"
		        }
		    }
		}
	    else if (imtype == "DARK") {
	        fib1 = "OFF"
	        fib2 = "OFF"
	        fib3 = "OFF"
		}
	    else if (imtype == "ZERO") {
	        fib1 = "OFF"
	        fib2 = "OFF"
	        fib3 = "OFF"
		}
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", filename, imtype, object, peri, bin1, fibsize, osswitch, exptime, comptime, fib1, fib2, fib3, >>tempkey)
	    if (vb)
		printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", filename, imtype, object, peri, bin1, fibsize, osswitch, exptime, comptime, fib1, fib2, fib3)
	    if (addfib) {
		hedit (imf, "FIB1", fib1, update+, verify-)
		hedit (imf, "FIB2", fib2, update+, verify-)
		hedit (imf, "FIB3", fib3, update+, verify-)
		}

	    if (listflat) {
		if (fibsize == "1.8") {
		    if (bin1 == "1") {
			if (fib1 == "FLAT")
			    print (filename, >> flats12
			else if (fib3 == "FLAT")
			    print (filename, >> flats3
			}
		    else {
			if (fib1 == "FLAT")
			    print (filename, >> flatsb12
			else if (fib3 == "FLAT")
			    print (filename, >> flatsb3
			}
		    }
		else if (fibsize == "2.3") {
		    if (bin1 == "1") {
			if (fib1 == "FLAT")
			    print (filename, >> flatm12
			else if (fib3 == "FLAT")
			    print (filename, >> flatm3
			}
		    else {
			if (fib1 == "FLAT")
			    print (filename, >> flatmb12
			else if (fib3 == "FLAT")
			    print (filename, >> flatmb3
			}
		    }
		}
	    if (listcomp) {
		if (fibsize == "1.8") {
		    if (bin1 == "1") {
			if (fib1 == "COMP")
			    print (filename, >> comps12
			else if (fib3 == "COMP")
			    print (filename, >> comps3
			}
		    else {
			if (fib1 == "COMP")
			    print (filename, >> compsb12
			else if (fib3 == "COMP")
			    print (filename, >> compsb3
			}
		    }
		else if (fibsize == "2.3") {
		    if (bin1 == "1") {
			if (fib1 == "COMP")
			    print (filename, >> compm12
			else if (fib3 == "COMP")
			    print (filename, >> compm3
			}
		    else {
			if (fib1 == "COMP")
			    print (filename, >> compmb12
			else if (fib3 == "COMP")
			    print (filename, >> compmb3
			}
		    }
		}
	    }
	}
    else if (vb)
	print ("TRESLIST: Using existing master keyword file ", tempkey)

    if (deltemp) {
	if (access (tempilist))
	    delete (tempilist)
	}
end
# Oct  1 2007	New program
