# QREF - Roughly dispersion correct a TRES calibration spectrum
# February 13, 2008
# By Doug Mink, SAO Telescope Data Center

	procedure qref (image)

string	image=""		{prompt="Calibration image to process"}
string	refdir="tresdata$"	{prompt="Reference image directory"}
string	refroot=""	{prompt="Reference image root name"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imin, impath, imroot, newfile, rdir, rroot
file	imout, reffile, refpath, imref
string	imroot, tsize, tbin, tfib
int	n
bool	vb, dt, sv

	vb = verbose
	dt = deltemp
	sv = saveold
	imin = image
	rdir = refdir
	rroot = refroot

	if (imin == "") {
	    print ("QREF: Roughly dispersion correct a TRES calibration spectrum")
	    print ("Usage: qref xxx.COMP_fibi.fits")
	    return
	    }

# If a number is given, prepend appropriate zeroes and wildcard
	isnum (imin)
	if (isnum.fpcode > 0) {
	    n = int (imin)
	    if (n < 10)
		imin = "000" // n // ".*"
	    else if (n < 100)
		imin = "00" // n // ".*"
	    else if (n < 1000)
		imin = "0" // n // ".*"
	    else
		imin = "0" // n // ".*"
	    }

# If file extension is missing, add wildcard
	else {
	    smatch (imin, ".fits")
	    if (smatch.found == no)
		imin = imin // "*"
	    }
	impath = imin
	filepath (impath)
	if (!access (filepath.files)) {
	    print ("QREF: Cannot read file ",filepath.files)
	    }
	filename (filepath.files)
	imin = filename.filename
	fileroot (imin)
	imref = fileroot.rootname
	fileroot (imref)
	imroot = fileroot.rootname

# Figure out aperture size, binning, and fibers present
	tidref.iddir = rdir
	tidref.idroot = rroot
	tidref.verbose = vb
	trsfib (imin, specdir="". verbose-, debug-)
	tfib = trsfib.fiber
	tsize = trsfib.apsize
	tsize = ""
	tbin = trsfib.binning

	pxcsao.spectra = ""
	pxcsao.specnum = "1-51"
	pxcsao.specband = 0
	pxcsao.specdir = ""
	pxcsao.correlate = "pixel"
	pxcsao.templates = ""
	pxcsao.tempnum = "1-51"
	pxcsao.tempband = 1
	pxcsao.tempdir = "tresdata$"
	pxcsao.echelle = yes
	pxcsao.st_lambda = INDEF
	pxcsao.end_lambda = INDEF
	pxcsao.obj_plot = no
	pxcsao.xcor_plot = no
	pxcsao.xcor_file = no
	pxcsao.fixbad = no
	pxcsao.badlines = "badlines.dat"
	pxcsao.s_emchop = "no"
	pxcsao.t_emchop = "no"
	pxcsao.s_abs_reject = 100.
	pxcsao.s_em_reject = 2.
	pxcsao.t_abs_reject = 0.
	pxcsao.t_em_reject = 0.
	pxcsao.bell_window = 0.05
	pxcsao.renormalize = no
	pxcsao.ncols = 4096
	pxcsao.interp_mode = "spline3"
	pxcsao.zeropad = "no"
	pxcsao.low_bin = 5
	pxcsao.top_low = 20
	pxcsao.top_nrun = 125
	pxcsao.nrun = 250
	pxcsao.vel_init = "zero"
	pxcsao.czguess = 0.
	pxcsao.nzpass = 0
	pxcsao.tshift = 0.
	pxcsao.svel_corr = "none"
	pxcsao.tvel_corr = "none"
	pxcsao.pkmode = 1
	pxcsao.pkfrac = 0.5
	pxcsao.pksrch = 25
	pxcsao.minvel = -100.
	pxcsao.maxvel = 100.
	pxcsao.report_mode = 2
	pxcsao.logfiles = "STDOUT,xcsao.log"
	pxcsao.save_vel = no
	pxcsao.rvcheck = no
	pxcsao.archive = no
	pxcsao.nsmooth = 0
	pxcsao.cvel = INDEF
	pxcsao.dvel = INDEF
	pxcsao.ablines = "ablines.dat"
	pxcsao.emlines = "emlines.dat"
	pxcsao.linedir = "rvsao$lib/"
	pxcsao.dispmode = 1
	pxcsao.displot = yes
	pxcsao.device = "stdgraph"
	pxcsao.vel_plot = "correlation"
	pxcsao.curmode = no
	pxcsao.hardcopy = no
	pxcsao.plotter = "stdplot"
	pxcsao.besttemp = ""
	pxcsao.velocity = 0.0
	pxcsao.velerr = 0.0
	pxcsao.r = 0.0
	pxcsao.temp_plot = no
	pxcsao.contsub_plot = no
	pxcsao.apodize_plot = no
	pxcsao.fft_plot = no
	pxcsao.tfft_plot = no
	pxcsao.uxcor_plot = no
	pxcsao.nsum = 1

# Find the shift between this spectrum and the reference spectrum with pxcsao
	tidref.refroot = rroot
	tidref.refdir = rdir
	tidref.verbose = vb
	tidref (imin)
	tempfile = tidref.reffile
	pxsao (imin, templates=tempfile)

# Make a new database dispersion file, including shift from template
	tdispref.refroot = rroot
	tdispref.refdir = rdir
	tdispref.verbose = vb
	tdispref (imin)
	reffile = tdispref.reffile
	refpath = dispref.refpath
	tdbmod.indir = rdir
	tdbmod.outdir = ""
	tdbmod.shift = 0.
	tdbmod.scale = 1.
	tdbmod.verbose = yes
	tdbmod.debug = no
	tdbmod (reffile, imin)

#  Reidentify spectrum features
	ecreidentify.images = ""
	ecreidentify.reference = ""
	ecreidentify.shift = 0.
	ecreidentify.cradius = 5.
	ecreidentify.threshold = 10.
	ecreidentify.refit = yes
	ecreidentify.database = "database"
	ecreidentify.logfiles = "STDOUT,logfile"
	ecreidentify (imin, imref)

# Add wavelength solution to spectra
	hedit (imin, "REFSPEC1", imref, add+, verify-)

# Do not linearize image twice
	imgetp.default = "0"
	imgetp.value = "0"
	imgetp (imin, "TDISP")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("QREF: ", imin, " already has a dispersion function")
		print ("TDISP = ", imgetp.value)
		}
	    return
	    }

# Dispersion-correct file
	if (vb) {
	    print (QREF: Set REFSPEC1 in ",imin," to ",imref)
	    }
	hedit (imin, "REFSPEC1", imref, verify=no,show=vb,add=yes,update=yes)
	imdisp = imroot // ".disp.ec.fits"
	dispcor.linearize = no
	dispcor.database = "database"
	dispcor.table = ""
	dispcor.w1 = INDEF
	dispcor.w2 = INDEF
	dispcor.dw = INDEF
	dispcor.nw = INDEF
	dispcor.log = no
	dispcor.flux = yes
	dispcor.blank = 0.
	dispcor.samedisp = no
	dispcor.global = no
	dispcor.ignoreaps = no
	dispcor.confirm = no
	dispcor.listonly = no
	dispcor.verbose = vb
	dispcor.logfile = ""
	dispcor (imin, imdisp, linearize=no)

# Save original file, if requested
	if (save) {
	    if (!access ("Nodisp/")) {
		mkdir ("Nodisp")
		}
	    newfile = "Nodisp/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Nodisp")
	    if (vb) {
		print ("QREF: Original ", imin, " => Nodisp/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename processed file to input file name
	rename (imdisp, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time
	history = history // " image dispersion from " // imref
        hedit (imin, "TDISP", history, add+, verify-, show-)

end

# Jan 25 2008	New task for quick look
# Jan 30 2008	Use new tasks to figure out reference spectra
# Feb 13 2008	Set specdir="" when calling trsfib
