# TAMPMERGE - Combine 2 amps into one TRES image
# February 13, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tampmerge (images)

string	images=""	{prompt="Image file(s) to merge"}

bool	saveold=yes	{prompt="Save unmerged files in Unmerged/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

struct	*inlist

begin

file	imroot, temp1, temp2, im4, imin, templist, imout
file	imfile, newfile, imex0
string	amp1, amp2, arg1, imx, ims, history, tchar
string	gain1, gain2
bool	dellist, vb

	vb = verbose
	imx = ".fits"
	ims = images

	if (ims == "") {
	    print ("TAMPMERGE: Combine 2 amps from 1 TRES image")
	    print ("Usage: tmerge <input image with 2 extensions>")
	    return
	    }

	hedit.add = no
	hedit.addonly = no
	hedit.delete = no
	hedit.verify = no
	hedit.show = no
	hedit.update = yes

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
            templist = mktemp ("tempmerge") // ".list"
            files (ims, >templist)
	    }
        inlist = templist
        while (fscan (inlist,imin) != EOF) {

	
# Do not merge amplifiers twice
	    imgetp.default = "0"
	    imex0 = imin // "[0]"
	    imgetp (imex0, "TAMPMERGE")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TAMPMERGE: ", imin, " has already been merged")
		    print ("TAMPMERGE = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		}
	    if (vb) {
		print ("TAMPMERGE: image root is ",imroot)
		}

#  Split out one image per amplifier
	    temp1 = mktemp ("tempamp1") // imx
	    amp1 = imroot // "[1]"
	    imcopy (amp1, temp1)
	    temp2 = mktemp ("tempamp2") // imx
	    amp2 = imroot // "[2]"
	    imcopy (amp2, temp2)

# Get gain correction terms for output header(s)
	    imgetp.default = "1.0"
	    imgetp (amp1, "GAINCORR")
	    gain1 = imgetp.value
	    imgetp.default = "1.0"
	    imgetp (amp2, "GAINCORR")
	    gain2 = imgetp.value

# Move original file to Unmerged directory
	    if (saveold) {
		if (!access ("Unmerged/")) {
		    mkdir ("Unmerged")
		    }
		newfile = "Unmerged/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (imin, "Unmerged")
		if (vb) {
		    print ("TAMPMERGE: Original ", imin, " => Unmerged/")
		    }
		}
	    else {
		delete (imin)
		}
            gettime ()

#  Join two amplifiers into a single image
	    arg1 = temp1 // "," // temp2 // "[-*,*]"
	    imjoin (arg1, imin, 1)

# Add gain corrections from each amplifier
	    hedit (imin, "GAINCORR", "", add-, delete+)
	    hedit (imin, "GAINCOR1", gain1, add+, delete-)
	    hedit (imin, "GAINCOR2", gain2, add+, delete-)

# Make history line including time for preamp correction
	    history = gettime.date // " " // gettime.time // " 2 amplifiers merged into one image"
	    hedit (imin, "TAMPMERGE", history, add+, delete-)

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		}
	    }
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
	inlist = ""
end

# Sep 28 2007	New task based on hectospec.hmerge

# Feb 11 2008	Change name of temporary file from mergelist* to tempmergelist*
# Feb 13 2008	Change temporary files from tamp* to tempamp*
