# File tres/Scripts/tcal1.cl
# April 2, 2008
# By Doug Mink

#  TCAL1 -- Make a local id database file for a calibration spectrum

procedure tcal1 (spectrum)

string  spectrum=""       {prompt="Images to check"}

string	refroot="comp"	{prompt="Root name for comparison reference file"}
string	refdir="tresdata$" {prompt="Directory for comparison reference file"}
real	shift=0.0	{prompt="Pixel shift (returned)"}
bool	interactive=no	{prompt="Run ecidentify on spectrum (yes or no)"}
bool	saveold = yes	{prompt="Save original file to Nodisp/ (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print extra information (yes or no)"}

begin

    string imin, calref, history, ecref, idref
    file imref, impix, newfile, rdir
    bool vb, db, ecid, sv, dt

    imin = spectrum
    rdir = refdir
    ecid = interactive
    dt = deltemp
    sv = saveold
    db = debug
    vb = verbose

# Skip this file if it has already been dispersion-corrected
    imgetp.default = "0"
    imgetp (imin, "TDISP")
    if (imgetp.value != "0") {
	if (vb) {
	    print ("TCAL1: ", imin, " has been dispersion-corrected")
	    print ("TDISP = ", imgetp.value)
	    }
	imgetp.default = "0"
	imgetp (imin, "TIDSHIFT")
	if (vb) {
	    print ("TIDSHIFT = ", imgetp.value)
	    }
	tcal1.shift = real (imgetp.value)
	return
	}

    tidref (imin, refroot=refroot, refdir=rdir, verbose=vb, debug=db)
    imref = tidref.reffile

    if (!deftask ("pxcsao")) {
	rvsao (motd=no)
	}

    pxcsao.spectra = imin
    pxcsao.specnum = "1-51"
    pxcsao.specband = 0
    pxcsao.specdir = ""
    pxcsao.correlate = "pixel"
    pxcsao.templates = imref
    pxcsao.tempnum = "1-51"
    pxcsao.tempband = 1
    pxcsao.tempdir = rdir
    pxcsao.echelle = yes
    pxcsao.st_lambda = INDEF
    pxcsao.end_lambda = INDEF
    pxcsao.obj_plot = no
    pxcsao.xcor_plot = no
    pxcsao.xcor_file = no
    pxcsao.fixbad = no
    pxcsao.badlines = "tharskip.dat"
    pxcsao.s_emchop = "no"
    pxcsao.t_emchop = "no"
    pxcsao.s_abs_reject = 100.
    pxcsao.s_em_reject = 2.
    pxcsao.t_abs_reject = 0.
    pxcsao.t_em_reject = 0.
    pxcsao.bell_window = 0.05
    pxcsao.renormalize = no
    pxcsao.ncols = 65536
    pxcsao.interp_mode = "sums"
    pxcsao.zeropad = "yes"
    pxcsao.low_bin = 5
    pxcsao.top_low = 20
    pxcsao.top_nrun = 65536
    pxcsao.nrun = 65536
    pxcsao.vel_init = "zero"
    pxcsao.czguess = 0.
    pxcsao.nzpass = 0
    pxcsao.tshift = 0.
    pxcsao.svel_corr = "none"
    pxcsao.tvel_corr = "none"
    pxcsao.pkmode = 3
    pxcsao.pkfrac = 0.95
    pxcsao.pksrch = 25
    pxcsao.minvel = -50.
    pxcsao.maxvel = 50.
    pxcsao.report_mode = 18
    pxcsao.logfiles = "STDOUT,xcsao.log"
    pxcsao.save_vel = no
    pxcsao.rvcheck = no
    pxcsao.archive = no
    pxcsao.nsmooth = 0
    pxcsao.cvel = INDEF
    pxcsao.dvel = INDEF
    pxcsao.ablines = ""
    pxcsao.emlines = "thar.dat"
    pxcsao.linedir = "tres$lib/"
    pxcsao.dispmode = 1
    pxcsao.displot = db
    pxcsao.device = "stdgraph"
    pxcsao.vel_plot = "correlation"
    pxcsao.curmode = no
    pxcsao.hardcopy = no
    pxcsao.plotter = "stdplot"
    pxcsao.besttemp = ""
    pxcsao.velocity = 0.0
    pxcsao.velerr = 0.0
    pxcsao.r = 0.0
    pxcsao.temp_plot = no
    pxcsao.contsub_plot = no
    pxcsao.apodize_plot = no
    pxcsao.fft_plot = no
    pxcsao.tfft_plot = no
    pxcsao.uxcor_plot = no
    pxcsao.nsum = 1
    pxcsao.debug = db

# Cross correlate with reference calibration spectrum
    pxcsao (imin)
    shift = pxcsao.velocity

# Copy reference database/ec file with shift from correlation
    tmodid.indir = rdir
    tmodid.outdir = ""
    tmodid.shift = pxcsao.velocity
    hedit (imin, "TIDSHIFT", pxcsao.velocity, add+, update+,verify-)
    tmodid.scale = 1.0
    if (interactive)
	tmodid.keepfit = no
    else
	tmodid.keepfit = yes
    tmodid.verbose = vb
    tmodid.debug = db
    tmodid (imref, imin)

# Adjust line fit interactively, if requested
    fileroot (imin)
    calref = fileroot.rootname
    hedit (imin, "REFSPEC1", calref, add+, update+,verify-)
    if (ecid) {
	ecidentify.coordlist = "linelists$thar.dat"
	ecidentify (imin)
	}

# Set dispersion task parameters
    dispcor.linearize = no
    dispcor.database = "database"
    dispcor.table = ""
    dispcor.w1 = INDEF
    dispcor.w2 = INDEF
    dispcor.dw = INDEF
    dispcor.nw = INDEF
    dispcor.log = no
    dispcor.flux = yes
    dispcor.blank = 0.
    dispcor.samedisp = no
    dispcor.global = no
    dispcor.ignoreaps = no
    dispcor.confirm = no
    dispcor.listonly = no
    dispcor.verbose = vb
    dispcor.logfile = ""

    fileroot (calref)
    impix = fileroot.rootname // "p.ec.fits"
    rename (imin, impix)
    dispcor (impix, imin, linearize=no)

# Add history line including time
    gettime ()
    history = gettime.date // " " // gettime.time // " dispersion fit from " // calref
    hedit (imin, "TDISP", history, add+, verify-)

# Move original file to Nodisp directory
    if (sv) {
	if (!access ("Nodisp/")) {
	    mkdir ("Nodisp")
	    }
	newfile = "Nodisp/" // imin
	if (access (newfile)) {
	    imdelete (newfile)
	    }
	rename (impix, newfile)
	if (vb) {
	    print ("TCAL1: Original ", imin, " => Nodisp/")
	    }
	}
    else if (dt) {
	delete (impix)
	}

end

# Jan 28 2008	New task
# Jan 29 2008	Add saveold to save undispersed spectrum to Nodisp/
# Jan 29 2008	Write TCAL FITS parameter to output file
# Mar 11 2008	Rename task TCAL1
# Mar 11 2008	Do not run task if the file is already dispersion-corrected
# Mar 15 2008	Add shift parameter to return computed shift for re-use
# Mar 15 2008	Add interactive parameter to make running ecidentify optional
# Apr  2 200	Fix bug by adding deltemp parameter
