# TFLAT1 -- Divide TRES image by pixel normalization file from tmakeflat
#          (apflatten), saving original file in Unflat/
# By Doug Mink, SAO Telescope Data Center
# March 18, 2008

procedure tflat1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
bool saveold = yes	{prompt="Save original file to Nonorm/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb, sv, makename
	file imin, imout, imroot, imnorm, newfile, templist, ndir
	int lroot
	string history, ims, tchar

	imin = image
	vb = verbose
	sv = saveold
	imnorm = normfile
	if (imnorm == "")
	    makename=yes
	else
	    makename = no
	ndir = normdir
	fixdir (ndir)
	ndir = fixdir.newdir
	if (ndir == "") {
	    ndir = "tresdata$"
	    }

# Do not flatten spectra twice
	imgetp.default = "0"
	imgetp (imin, "TFLAT")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TFLAT1: ", imin, " has been processed")
		print ("TFLAT = ", imgetp.value)
		}
	    return
	    }

# Do not try to flatten DARK or BIAS image with no fiber data
	imgetp.default = "0"
	imgetp (imin, "FIBKEY")
	if (imgetp.value == "NNN") {
	    if (vb) {
		print ("TFLAT1: ", imin, " cannot be flattened")
		}
	    return
	    }

# Remove extension from input file, if one is present, and make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".flattened.fits"
	    }
	else {
	    imout = imin // ".flattened.fits"
	    imin = imin // ".fits"
	    }
	if (makename) {
	    trsfib (imin, specdir="")
	    imnorm = ndir // "flat" // trsfib.apsize // trsfib.binning
	    if (trsfib.fib1 != "OFF")
		imnorm = imnorm // "1"
	    if (trsfib.fib2 != "OFF")
		imnorm = imnorm // "2"
	    if (trsfib.fib3 != "OFF")
		imnorm = imnorm // "3"
	    imnorm = imnorm // ".flat.fits"
	    }

# Add FITS extension to normalization file if it is missing
	else {
	    smatch (imnorm, ".fits")
	    if (!smatch.found) {
		imnorm = imnorm // ".fits"
		}
	    }

	if (vb) {
	    print ("TFLAT1: ", imin," -> ",imout," using ", imnorm)
	    }

# Divide out normalization spectra
	imarith (imin, "/", imnorm, imout)

# Move original file to Unflat directory
	if (sv) {
	    if (!access ("Unflat/")) {
		mkdir ("Unflat")
		}
	    newfile = "Unflat/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Unflat")
	    if (vb) {
		print ("TFLAT1: Original ", imin, " => Unflat/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " pixels normalized by " // imnorm
        hedit (imin, "TFLAT", history, add+, verify-)

end

# Feb 22 2008	New task separated from TFLAT
# Mar 18 2008	Fix bug: argument mismatch images/image
