# TMAKEFLAT -- Run apflatten to make a 2-d pixel normalization file
# By Doug Mink, SAO Telescope Data Center
# February 8, 2008

procedure tmakeflat (flatfile)

string	flatfile = ""	{prompt="Input image or list to combine and fit"}
file	imdir=""	{prompt="Directory for flat and norm files"}

string	apertures="1-104" {prompt="Apertures to flatten"}
string	apref=""	{prompt="Aperture reference file"}
int	norders=51	{prompt="Number of orders to use"}
int	aporder=5	{prompt="Order to fit apertures"}
file	apdir=""	{prompt="Aperture reference directory"}
bool	verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imflat, imnorm, idir, aref, adir, normfile
	file sumfile, listfile, dbfile
	int nap
	string ffile, tchar, history

	ffile = flatfile
	fixdir (imdir)
	idir = imdir
	imflat = idir // ffile
	vb = verbose
	nap = norders

# If input image is list file, process the images
	tchar = substr (ffile,1,1)
	if (tchar == "@") {
	    tproc (ffile, cosmic+, flatten-, extract-, disperse-)
	    }

# If input image is list file, add it together
	if (tchar == "@") {
	    listfile = substr (ffile,2,100)
	    fileroot (listfile)
	    sumfile = fileroot.rootname // ".fits"
	    if (!access (sumfile)) {
		trssum (ffile, sumfile)
		if (vb)
		    print ("TMAKEFLAT: ",sumfile," summed from ",ffile)
		}
	    else if (vb)
		print ("TMAKEFLAT: ",sumfile," already exists")
	    ffile = sumfile
	    }

# Generate output file name from input file
	fileroot (ffile)
	normfile = fileroot.rootname // ".flat.fits"
	imnorm = idir // normfile

# Add extension to input file, if one is not present
	smatch (ffile, ".fits")
	if (!smatch.found) {
	    ffile = ffile // ".fits"
	    }

	if (vb) {
	    print ("TMAKEFLAT: ", ffile," -> ",imnorm)
	    }

	if (access (imnorm))
	    delete (imnorm)

# Find and trace apertures (=orders) in image, if not already done
	fileroot (ffile)
	dbfile = "database/ap" // fileroot.rootname
	if (!access (dbfile)) {
	    smatch (ffile, "12")
	    if (smatch.found) {
		apfind (ffile, nap*2)
		}
	    else {
		apfind (ffile, nap)
		}
	    apedit (ffile)
	    aptrace (ffile)
	    }

# Set reference file and directory
	aref = apref
	if (aref == "") {
	    aref = ffile
	    adir = idir
	    }
	else {
	    fixdir (apdir)
	    adir = fixdir.newdir
	    }

# Run apflatten
	apflatten.apertures = apertures
	apflatten.references = adir // aref
	apflatten.interactive = yes
	apflatten.find = no
	apflatten.recenter = no
	apflatten.resize = no
	apflatten.edit = no
	apflatten.trace = no
	apflatten.fittrace = no
	apflatten.flatten = yes
	apflatten.fitspec = yes
	apflatten.line = INDEF
	apflatten.nsum = 1
	apflatten.threshold = 10.
	apflatten.pfit = "fit1d"
	apflatten.clean = no
	apflatten.saturation = INDEF
	apflatten.readnoise = "2.8"
	apflatten.gain = "1."
	apflatten.lsigma = 4.
	apflatten.usigma = 4.
	apflatten.function = "spline3"
	apflatten.order = aporder
	apflatten.sample = "*"
	apflatten.naverage = 1
	apflatten.niterate = 0
	apflatten.low_reject = 3.
	apflatten.high_reject = 3.
	apflatten.grow = 0.
	apflatten (ffile, imnorm)

# Add history line including time to normalization file
	gettime ()
	history = gettime.date // " " // gettime.time // " order " // aporder // " fit to "// ffile
        hedit (imnorm, "PIXFLAT", history, add+, verify-, update+, show-)
	if (vb)
	    print ("TMAKEFLAT: ",history)
	return
end
# Oct  2 2007	New task
# Oct 10 2007	Assign flatfile to ffile so there is only one query

# Jan 28 2008	Add aperture/order finding and tracing
# Jan 31 2008	Add input file summation (cosmic rays assumed to be removed)
# Feb  5 2008	Add dispers- to tproc invocation
# Feb  8 2008	Add return at end
