# File tres/Scripts/tmodid.cl
# By Doug Mink
# March 20, 2008

# TMODID -- Modify line positions in ecidentify database file

procedure tmodid (dbfin, dbfout)

string dbfin=""		{prompt="File to process"}
string dbfout=""	{prompt="File for output"}

string indir="tresdata$" {prompt="Input directory with database directory"}
string outdir=""	{prompt="Output directory with database directory"}
real shift=0.0		{prompt="Constant pixel shift (0 is none)"}
real scale=1.0		{prompt="Constant pixel scale factor (1 is none)"}
bool keepfit=no		{prompt="Keep dispersion fit (yes or no)"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print output (yes or no)"}

begin

    bool vb, db, kf
    int i, io
    file idbpath, idbdir, odbpath, odbdir
    string idbfile, odbfile, idbroot, odbroot
    string s1, s2, s3, s4, s5, s6, s7, s8
    real pshift, pscale, x, w1, w2

    vb = verbose
    db = debug
    kf = keepfit
    pshift = shift
    pscale = scale

# Input dispersion fit file
    if (indir != "") {
	fixdir (indir)
	idbdir = fixdir.newdir // "database/"
	}
    else {
	idbdir = "database/"
	}
    idbfile = dbfin
    fileroot (idbfile)
    idbroot = fileroot.rootname
    idbpath = idbdir // "ec" // idbroot

# Output dispersion fit file
    if (outdir != "") {
	fixdir (outdir)
	odbdir = fixdir.newdir // "database/"
	}
    else {
	odbdir = "database/"
	}
    odbfile = dbfout
    fileroot (odbfile)
    odbroot = fileroot.rootname
    odbpath = odbdir // "ec" // odbroot

# Make sure database directory is there
    if (!access (idbpath)) {
	print ("TMODID: Database file ",idbpath, " does not exist")
	return
	}
    if (vb) {
	print ("TMODID: Modifying ",idbpath," to ", odbpath)
	}
    s1 = ""
    s2 = ""
    s3 = ""
    s4 = ""
    s5 = ""
    s6 = ""
    s7 = ""
    s8 = ""
    gettime (usesec+)

# Read through database entry one line at a time
    list = idbpath
    while (fscan (list, s1, s2, s3, s4, s5, s6, s7, s8) != EOF) {
	    if (s1 == "") {
		printf ("\n", >>odbpath)
	    if (db) {
		printf ("\n")
		}
	    }
	else if (s1 == "#") {
	    printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time, >>odbpath)
	    if (db) {
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time)
		}
	    }
	else if (s3 == "") {
	    if (s1 == "id") {
		printf ("\tid\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\tid\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "image") {
		printf ("\timage\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\timage\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "slope") {
		printf ("\tslope\t%s\n", s2, >>odbpath)
		if (db) {
		    printf ("\tslope\t%s\n", s2)
		    }
		if (!keepfit) {
		    break
		    }
		}
	    else {
		printf ("\t%s\t%s\n", s1, s2, >>odbpath)
		if (db) {
		    printf ("\t%s\t%s\n", s1, s2)
		    }
		}
	    }
	else if (s4 == "") {
	    printf ("begin\tecidentify %s\n", odbroot, >>odbpath)
	    if (db) {
		printf ("begin\tecidentify %s\n", odbroot)
		}
	    }
	else {
	    if (keepfit) {
		x = real (s3)
		}
	    else {
		x = (real (s3) * pscale) + pshift
		}
	    w1 = real (s4)
	    w2 = real (s5)
	    i = int (s1)
	    io = int (s2)
	    printf ("\t\t%2d %4d %8.2f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8, >>odbpath)
	    if (db) {
		printf ("\t\t%2d %4d %8.2f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8)
		}
	    }
	s1 = ""
	s2 = ""
	s3 = ""
	s4 = ""
	s5 = ""
	s6 = ""
	s7 = ""
	s8 = ""
	}
end

# Jan 22 2008	New program to copy a database echelle dispersion file
# Jan 25 2008	Add debugging code; break after slope line
# Jan 28 2008	Add keepfit parameter to simply copy database entry
# Feb  1 2008	Print processing date and time instead of original date and time
# Mar 20 2008	Null out all strings after each line is read
