# TPREAMP1 - Run SPP version of Brian McCleod's preamplifier fix on raw image
# March 25, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tpreamp1 (image)

string	image=""	{prompt="Images to process"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}
bool	debug=no	{prompt="Print more process progress (yes or no)"}

begin

file	imroot, temp1, imin, imout, rawdir
string	imx, imsec, history
string	pfsect0, pfsect1, pfsect2
bool	vb, dt, db, sv

	vb = verbose
	dt = deltemp
	db = debug
	imx = ".fits"
	imin = image
	sv = saveold

	if (imin == "") {
	    print ("TPREAMP: Correct preamp problem for all four amps one amp at a time")
	    print ("Usage: hpreamp <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (sv) {
	    rawdir = "Unpreamp/"
	    if (!access (rawdir))
		mkdir (rawdir)
	    }

# Do not process images without extensions
	imgetp.default = "-1"
	imsec = imin // "[0]"
	imgetp (imsec, "i_naxis")
	if (imgetp.value != "0") {
	    print ("TPREAMP1: ", imin, " is not a multi-extension image")
	    print ("NAXIS = ", imgetp.value)
	    return
	    }

# Do not process image twice
	imgetp.default = "0"
	imsec = imin // "[1]"
	imgetp (imsec, "PREAMPFX")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TPREAMP1: ", imin, " has already been processed")
		print ("PREAMPFX = ", imgetp.value)
		}
	    return
	    }

# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imin
	    imin = imin // imx
	    }

# Fix preamp output using SPP version of Brian McLeod's C program preampfix
	if (db) {
	    print ("TPREAMP1: Making preamp correction for ",imin)
	    print ("TPREAMP1: image root is ",imroot)
	    }

# Output filename is the same as the input file name
	tampfix.verbose = vb
	tampfix.debug = db
	tampfix.error = ""
	tampfix.indir = ""
	pfsect0 = imroot // "_0.fits"
	imcopy (imin // "[0]", pfsect0, verbose=vb)
	pfsect1 = imroot // "_1.fits"
	imcopy (imin // "[1]", pfsect1, verbose=vb)
	tampfix (pfsect1)
	if (tampfix.error != "")
	    print ("TAMPFIX 1: ",tampfix.error)
	pfsect2 = imroot // "_2.fits"
	imcopy (imin // "[2]", pfsect2, verbose=vb)
	tampfix.error = ""
	tampfix (pfsect2)
	if (tampfix.error != "")
	    print ("TAMPFIX 2: ",tampfix.error)

# Save unchanged image
	if (sv) {
	    imout = "Unpreamp/" // imin
	    if (!access (imout))
		rename (imin, imout)
	    else
		delete (imin)
	    }
	else
	    delete (imin)

# Reassemble the multi-extension FITS file
	if(vb)
	    print("TPREAMP: rejoining ",imin)

# Reassemble the input file with the fixed extensions
	mscjoin (imroot, output=imin, delete=dt, verbose=vb)

	hedit (imsec, "PREAMPFX",imgetp.value,update+,verify-,show=vb)

	return
end

# Feb 25 2008	New task extracted from TFLAT
# Feb 25 2008	Drop output directory parameter; it was not being used
# Mar 25 2008	Fix argument images->image
