# TPROC - Run image processing tasks on both TRES amps, combine, and flatten
# April 4, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=no	{prompt="Make preamp correction (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	flatten=yes	{prompt="Run tflat on each image (yes or no)"}
bool	cosmic=yes	{prompt="Remove cosmic rays from a list of images (yes or no)"}
bool	extract=yes	{prompt="Extract spectra using textract (yes or no)"}
bool	disperse=yes	{prompt="Add wavelength dispersion function to spectrum (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$"     {prompt="Directory for normalization file"}
char    sumspec="no"    {prompt="Process summed 2-D file after tcosmic (yes no only)"}
string	aproot="flat"	{prompt="Root filename for aperature reference spectrum"}
string	apdir="tresdata$"	{prompt="Directory for reference spectrum"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	refroot="comp"	{prompt="Root filename for wavelength reference spectrum"}
string	refdir="tresdata$"	{prompt="Directory for reference spectrum"}
real	refshift=0.	{prompt="Pixel shift from reference comparison spectrum"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *procfiles
begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning, nd, nf, temparg, apd, apr
bool	vb, dt, sv, fp, fb, flat, disp, recent, fpr, svr, sum, sumonly
int	nfiles
real	gc, tshift1, tshift2

	vb = verbose
	dt = deltemp
	sv = saveold
	svr = saveraw
	fp = fixpix
	fpr = fixpreamp
	fb = fixbias
	flat = flatten
	recent = recenter
	disp = disperse
	gc = gaincorr
	nf = normfile
	nd = normdir
	apr = aproot
	apd = apdir
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPROC: Process TRES image files to extraction")
	    print ("Usage: tproc im1,im2,...,imn")
	    print ("Usage: tproc *.fits")
	    print ("Usage: tproc @image_list_file")
	    return
	    }

	sum = no
	sumonly = no
	if (sumspec == "yes") {
	    sum = yes
	    }
	else if (sumspec == "only") {
	    sumonly = yes
	    sum = yes
	    }

	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempproc") // ".list"
	if (tchar == '@') {
	    imlist = substr (ims,2,100)
	    copy (imlist, templist)
	    }
	else {
	    files (ims, >templist)
	    }

# Read through input files
	procfiles = templist
	nfiles = 0
        while (fscan (procfiles,imin) != EOF) {

# Fix preamplifier problem which puts flux in overscan region
	    if (fixpreamp)
		tpreamp1 (imin,deltemp=dt,saveold=sv,verbose=vb)

# Process files as images
	    tproc1 (imin,fixbias=fb,fixpix=fp,deltemp=dt,saveraw=svr,saveold=sv,verbose=vb)

# Correct gain differences between amplifiers
	    tgain1 (imin,g2=gc,deltemp=dt,saveold=sv,verbose=vb)

# Merge amplifiers into a single image
	    tampmerge (imin,saveold=sv,deltemp=dt,verbose=vb)

# Flatfield correct spectra
	    if (flat)
		tflat1 (imin,normfile=nf,normdir=nd,saveold=sv,verbose=vb)
	    nfiles = nfiles + 1
	    }

# Remove cosmic rays, if requested
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		tcosmic (temparg, savebp=sv, saveold=sv, deltemp=dt, verbose=vb)
		}
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }
	if (sum) {
	    temparg = "@" // templist
	    trssum (temparg)
	    if (sumonly) {
		print (trssum.outfile, >templist)
		}
	    else {
		print (trssum.outfile, >>templist)
		}
	    }

# Extract 1D x orders spectrum and add dispersion, if requested
	if (extract) {
	    procfiles = templist
	    tshift1 = 0.0
	    tshift2 = 0.0
            while (fscan (procfiles,imin) != EOF) {
		textract1 (imin, aproot=apr, apdir=apd, recenter=recent, verbose=vb)
		if (disp) {
		    if (textract1.spec3 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec3, "FIB3")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec3, interactive=no, saveold=sv, deltemp=dt, verbose=vb)
			    tshift1 = tcal1.shift
			    tshift2 = tcal1.shift
			    }
			else
			    tdisp1 (textract1.spec3, saveold=sv, verbose=vb)
			}
		    if (textract1.spec1 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec1, "FIB1")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec1, interactive=no, saveold=sv, deltemp=dt, verbose=vb)
			    tshift1 = tcal1.shift
			    }
			else
			    tdisp1 (textract1.spec1, shift=tshift1, saveold=sv, verbose=vb)
			}
		    if (textract1.spec2 != "") {
			imgetp.default = "NONE"
			imgetp (textract1.spec2, "FIB2")
			if (imgetp.value == "COMP") {
			    tcal1.shift = 0.0
			    tcal1 (textract1.spec2, interactive=no, saveold=sv, verbose=vb)
			    tshift2 = tcal1.shift
			    }
			else
			    tdisp1 (textract1.spec2, shift=tshift2, saveold=sv, verbose=vb)
			}
		    }
		}
	    }

	if (dt)
	    delete (templist)
	procfiles = ""
end

# Jan 09 2008	New program to process list of TRES images
# Jan 17 2008	Update normfile and normdir prompts
# Jan 24 2008	Add optional bad pixel file correction with fixpix parameter
# Jan 28 2008	Add optional cosmic ray removal from list of images with TCOSMIC
# Jan 31 2008	Add optional spectrum extraction from list of images
# Feb  8 2008	Add saveraw to save raw images but not others
# Feb  8 2008	Add optional wavelength dispersion using TDISP
# Feb 11 2008	Change temporary list file from temproclist* to tempproc*list
# Feb 12 2008	Add recenter parameter for extraction
# Feb 25 2008	Use TEXTRACT1 to process images one at a time, not TEXTRACT
# Feb 25 2008	Use TPROC1 to process images one at a time, not TPREPROC
# Feb 25 2008	Call TPREAMP1 and TGAIN1 instead of using TPREPROC
# Feb 25 2008	Change name of TDISP to TDISP1 for consistency
# Mar 18 2008	Drop deltemp from arguments to TFLAT1
# Apr 10 2008	Drop deltemp from arguments to TDISP1
# Mar 18 2008	Add TAMPMERGE
# Mar 19 2008	Run TCAL1 on COMP spectra instead of TDISP1
# Mar 26 2008	Pass saveraw to TPROC1
# Mar 31 2008	Set dispersion pixel shift from most recent calibration lamp
# Apr  2 2008	Add deltemp, saveold, and verbose arguments to tcosmic and tcal1
# Apr  2 2008	Add deltemp and verbose arguments to textract1 and tdisp1
# Apr  4 2008	Add sumspec parameter to process summed spectra
