# File tres/Scripts/trssum.cl
# April 10, 2008
# By Doug Mink, SAO Telescope Data Center

# TRSSUM - Use mscred.combine to add together a list of images

procedure trssum (images)

string	images=""	{prompt="Images to combine"}
int	sumseq=201	{prompt="Sequence number to use for output file"}
file	outfile=""	{prompt="Output file name (returned)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool    saveold=yes     {prompt="Save original image files to Uncombined/ (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	templist, imout, imin1
string	tchar, imagelist, history, dstring, outroot, fn1, fn2
file	listfile, imfile
bool	dellist, vb, done
int	nfiles

	vb = verbose
	dellist = deltemp
	imagelist = images

# Load needed packages if necessary
	if (!defpac ("mscred"))
	    mscred

#  Make temporary list of input files
	templist = mktemp ("tempcombine") // ".list"
	files (imagelist, >templist)

# Count files and initialize output root name
	nfiles = 0
	done = yes
	list = templist
	while (fscan (list,imin1) != EOF) {
	    nfiles = nfiles + 1
	    if (nfiles == 1) {
		smatch (imin1, ".fits")
		if (smatch.found) {
		    outroot = substr (imin1, 6, smatch.start)
		    }
		else {
		    outroot = substr (imin1, 6, 100)
		    }
		fn1 = substr (imin1,1,4)
		}
	    fn2 = substr (imin1,1,4)
	    }
	if (nfiles == 0) {
	    print ("TRSSUM: No files to process")
	    return
	    }
	else if (nfiles == 1) {
	    print ("TRSSUM: Only one file, returning it as sum")
	    }
	else if (vb) {
	    print ("TRSSUM is adding ",nfiles," files")
	    }
	imout = sumseq // "." // outroot // "_sum_" // fn1 // "_" // fn2 // ".fits"
	if (nfiles == 1) {
	    copy (imin1, imout)
	    return
	    }

	combine.input = "@" // templist
	combine.output = imout
	combine.headers = ""
	combine.bpmasks = ""
	combine.rejmasks = ""
	combine.nrejmasks = ""
	combine.expmasks = ""
	combine.sigmas = ""
	combine.ccdtype = ""
	combine.amps = no
	combine.subsets = no
	combine.delete = no
	combine.combine = "sum"
	combine.reject = "none"
	combine.project = no
	combine.outtype = "real"
	combine.outlimits = ""
	combine.offsets = "none"
	combine.masktype = "none"
	combine.maskvalue = 0.
	combine.blank = 0.
	combine.scale = "none"
	combine.zero = "none"
	combine.weight = "none"
	combine.statsec = ""
	combine.lthreshold = INDEF
	combine.hthreshold = INDEF
	combine.nlow = 1
	combine.nhigh = 1
	combine.nkeep = 1
	combine.mclip = yes
	combine.lsigma = 3.
	combine.hsigma = 3.
	combine.rdnoise = "0."
	combine.gain = "1."
	combine.snoise = "0."
	combine.sigscale = 0.1
	combine.pclip = -0.5
	combine.grow = 0.

	combine (imagelist, imout)


# Make history line including time
	gettime ()
	dstring = gettime.date // " " // gettime.time
	history = dstring // " images summed"
	hedit (imout, "TRSSUM", history, add+, verify-)
	outfile = imout

# Increment sequence number for this directory
	sumseq = sumseq + 1

	if (dellist) {
	    if (access (templist))
		delete (templist)
	    }

end

# Dec 13 2007	New program for TRES pipeline

# Jan 10 2008	Change name from tressum to trssum
# Jan 31 2008	Add TRSSUM keyword to output header
# Apr 10 2008	Create output file name from input list and sequence number
